@@ -453,7 +453,7 @@ setup_keys_by_type (SeahorseContext *sctx, SeahorseKey *skey, gboolean add)
 {
     GList *l, *keys = NULL;
     SeahorseKey *akey, *next;
-    gpointer kt = (gpointer)seahorse_key_get_keyid (skey);
+    gpointer kt = GUINT_TO_POINTER (seahorse_key_get_keyid (skey));
     gboolean first;
     
     /* Get current set of keys in this ktype/keyid */
@@ -463,7 +463,7 @@ setup_keys_by_type (SeahorseContext *sctx, SeahorseKey *skey, gboolean add)
     for (akey = g_hash_table_lookup (sctx->pv->keys_by_type, kt); 
          akey; akey = akey->preferred)
     {
-        if(akey != skey)
+        if (akey != skey)
             keys = g_list_prepend (keys, akey);
     }
     
@@ -596,7 +596,7 @@ seahorse_context_find_key (SeahorseContext *sctx, GQuark keyid, SeahorseKeyLoc l
         if (location == SKEY_LOC_INVALID && !skey->preferred)
             return skey;
         
-        if (location == seahorse_key_get_location (skey))
+        if (location >= seahorse_key_get_location (skey))
             return skey;
         
         /* Look down the list for this location */
@@ -650,6 +650,7 @@ seahorse_context_remove_key (SeahorseContext *sctx, SeahorseKey *skey)
     if (g_hash_table_lookup (sctx->pv->keys_by_source, k)) {
         g_object_ref (skey);
         g_signal_handlers_disconnect_by_func (skey, key_changed, sctx);
+        g_signal_handlers_disconnect_by_func (skey, key_destroyed, sctx);
         g_hash_table_remove (sctx->pv->keys_by_source, k);
         setup_keys_by_type (sctx, skey, FALSE);
         g_signal_emit (sctx, signals[REMOVED], 0, skey);    
@@ -846,7 +847,7 @@ refresh_keyservers (GConfClient *client, guint id, GConfEntry *entry,
 
 SeahorseOperation*
 seahorse_context_transfer_keys (SeahorseContext *sctx, GList *keys, 
-                                SeahorseKeySource *to)
+                                SeahorseKeySource *to, gboolean individually)
 {
     SeahorseKeySource *from;
     SeahorseOperation *op = NULL;
@@ -862,9 +863,14 @@ seahorse_context_transfer_keys (SeahorseContext *sctx, GList *keys,
     keys = seahorse_util_keylist_sort (keys);
     
     while (keys) {
+        
+        /* When individual, break off each one */
+        if (individually)
+            next = g_list_remove_link (keys, keys);
      
-        /* Break off one set (same keysource) */
-        next = seahorse_util_keylist_splice (keys);
+        /* Otherwise break off one set (same keysource) */
+        else 
+            next = seahorse_util_keylist_splice (keys);
 
         g_assert (SEAHORSE_IS_KEY (keys->data));
         skey = SEAHORSE_KEY (keys->data);
@@ -1028,7 +1034,7 @@ seahorse_context_discover_keys (SeahorseContext *sctx, GQuark ktype,
     
     /* Start an import process on all toimport */
     if (toimport) {
-        op = seahorse_context_transfer_keys (sctx, toimport, NULL);
+        op = seahorse_context_transfer_keys (sctx, toimport, NULL, FALSE);
         
         g_list_free (toimport);
         
