@@ -31,7 +31,7 @@ seahorse_key_predicate_match (SeahorseKeyPredicate *kl, SeahorseKey *skey)
     /* Now go for all the fields */
     if (kl->ktype && (kl->ktype != skey->ktype))
         return FALSE;
-    if (kl->keyid && (!skey->keyid || !g_str_equal (kl->keyid, skey->keyid)))
+    if (kl->keyid && (kl->keyid != skey->keyid))
         return FALSE;
     if (kl->location && (kl->location != skey->location))
         return FALSE;
@@ -50,7 +50,7 @@ seahorse_key_predicate_match (SeahorseKeyPredicate *kl, SeahorseKey *skey)
 }
 
 enum {
-	PROP_0,
+    PROP_0,
     PROP_KEY_SOURCE,
     PROP_KEY_ID,
     PROP_KTYPE,
@@ -60,102 +60,40 @@ enum {
     PROP_LOADED
 };
 
+/* Special fields */
 enum {
-	CHANGED,
-	LAST_SIGNAL
+    PROP_SPECIAL_DISPLAY_NAME,
+    PROP_SPECIAL_CN,
+    PROP_SPECIAL_MAX
+};
+
+GQuark special_properties[PROP_SPECIAL_MAX] = { 0 };
+
+enum {
+    CHANGED,
+    LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
-static GObjectClass *parent_class = NULL;
 
-/* GObject handlers */
-static void class_init        (SeahorseKeyClass *klass);
-static void object_finalize   (GObject *gobject);
+G_DEFINE_TYPE (SeahorseKey, seahorse_key, GTK_TYPE_OBJECT);
 
-static void set_property      (GObject *object, guint prop_id, 
-                               const GValue *value, GParamSpec *pspec);
-static void get_property      (GObject *object, guint prop_id,
-                               GValue *value, GParamSpec *pspec);
-
-GType
-seahorse_key_get_type (void)
-{
-	static GType gtype = 0;
-	
-	if (!gtype) {
-		static const GTypeInfo gtinfo = {
-			sizeof (SeahorseKeyClass), NULL, NULL,
-			(GClassInitFunc) class_init, NULL, NULL,
-			sizeof (SeahorseKey), 0, NULL
-		};
-		
-		gtype = g_type_register_static (GTK_TYPE_OBJECT, "SeahorseKey", 
-                                        &gtinfo, 0);
-	}
-	
-	return gtype;
-}
+/* -----------------------------------------------------------------------------
+ * OBJECT 
+ */
 
 static void
-class_init (SeahorseKeyClass *klass)
+seahorse_key_init (SeahorseKey *skey)
 {
-    GObjectClass *gobject_class;
-	
-    parent_class = g_type_class_peek_parent (klass);
-    gobject_class = G_OBJECT_CLASS (klass);
-	
-    gobject_class->finalize = object_finalize;
-    gobject_class->set_property = set_property;
-    gobject_class->get_property = get_property;
-	
-	klass->changed = NULL;
-	
-    g_object_class_install_property (gobject_class, PROP_KEY_SOURCE,
-        g_param_spec_object ("key-source", "Key Source", "Key Source that this key belongs to", 
-                             SEAHORSE_TYPE_KEY_SOURCE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
-    g_object_class_install_property (gobject_class, PROP_KEY_ID,
-        g_param_spec_string ("key-id", "Key ID", "Key identifier", 
-                             "UNKNOWN ", G_PARAM_READABLE));
     
-    g_object_class_install_property (gobject_class, PROP_KTYPE,
-        g_param_spec_uint ("ktype", "Key Type", "Key type", 
-                           0, G_MAXUINT, SKEY_INVALID, G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class, PROP_ETYPE,
-        g_param_spec_uint ("etype", "Encrpyption Type", "Key encryption type", 
-                           0, G_MAXUINT, SKEY_INVALID, G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class, PROP_FLAGS,
-        g_param_spec_uint ("flags", "Key Flags", "Flags on capabilities of key. See SeahorseKeyFlags", 
-                           0, G_MAXUINT, 0, G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class, PROP_LOCATION,
-        g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseKeyLoc", 
-                           0, G_MAXUINT, SKEY_LOC_UNKNOWN, G_PARAM_READABLE));
-
-    g_object_class_install_property (gobject_class, PROP_LOADED,
-        g_param_spec_uint ("loaded", "Loaded Information", "Which parts of the key are loaded. See SeahorseKeyLoaded", 
-                           0, G_MAXUINT, SKEY_INFO_NONE, G_PARAM_READABLE));
-
-	signals[CHANGED] = g_signal_new ("changed", G_OBJECT_CLASS_TYPE (gobject_class),
-		G_SIGNAL_RUN_LAST,  G_STRUCT_OFFSET (SeahorseKeyClass, changed),
-		NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
-}
-
-/* Unrefs gpgme key and frees data */
-static void
-object_finalize (GObject *gobject)
-{
-	G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 static void
-set_property (GObject *object, guint prop_id, const GValue *value, 
-              GParamSpec *pspec)
+seahorse_key_set_property (GObject *object, guint prop_id, const GValue *value, 
+                           GParamSpec *pspec)
 {
-	SeahorseKey *skey = SEAHORSE_KEY (object);
-	
+    SeahorseKey *skey = SEAHORSE_KEY (object);
+    
     switch (prop_id) {
     case PROP_KEY_SOURCE:
         g_assert (!skey->sksrc);
@@ -167,17 +105,17 @@ set_property (GObject *object, guint prop_id, const GValue *value,
 }
 
 static void
-get_property (GObject *object, guint prop_id, GValue *value, 
-              GParamSpec *pspec)
+seahorse_key_get_property (GObject *object, guint prop_id, GValue *value, 
+                           GParamSpec *pspec)
 {
     SeahorseKey *skey = SEAHORSE_KEY (object);
-	
+    
     switch (prop_id) {
     case PROP_KEY_SOURCE:
         g_value_set_object (value, skey->sksrc);
         break;
     case PROP_KEY_ID:
-        g_value_set_string (value, skey->keyid);
+        g_value_set_uint (value, skey->keyid);
         break;
     case PROP_KTYPE:
         g_value_set_uint (value, skey->ktype);
@@ -196,6 +134,67 @@ get_property (GObject *object, guint prop_id, GValue *value,
     }
 }
 
+static void
+seahorse_key_object_finalize (GObject *gobject)
+{
+    G_OBJECT_CLASS (seahorse_key_parent_class)->finalize (gobject);
+}
+
+static void
+seahorse_key_class_init (SeahorseKeyClass *klass)
+{
+    GObjectClass *gobject_class;
+    
+    seahorse_key_parent_class = g_type_class_peek_parent (klass);
+    gobject_class = G_OBJECT_CLASS (klass);
+    
+    gobject_class->finalize = seahorse_key_object_finalize;
+    gobject_class->set_property = seahorse_key_set_property;
+    gobject_class->get_property = seahorse_key_get_property;
+    
+    klass->changed = NULL;
+    
+    g_object_class_install_property (gobject_class, PROP_KEY_SOURCE,
+        g_param_spec_object ("key-source", "Key Source", "Key Source that this key belongs to", 
+                             SEAHORSE_TYPE_KEY_SOURCE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property (gobject_class, PROP_KEY_ID,
+        g_param_spec_uint ("key-id", "Key ID", "Key identifier", 
+                           0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
+    
+    g_object_class_install_property (gobject_class, PROP_KTYPE,
+        g_param_spec_uint ("ktype", "Key Type", "Key type", 
+                           0, G_MAXUINT, SKEY_UNKNOWN, G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class, PROP_ETYPE,
+        g_param_spec_uint ("etype", "Encrpyption Type", "Key encryption type", 
+                           0, G_MAXUINT, SKEY_ETYPE_NONE, G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class, PROP_FLAGS,
+        g_param_spec_uint ("flags", "Key Flags", "Flags on capabilities of key. See SeahorseKeyFlags", 
+                           0, G_MAXUINT, 0, G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class, PROP_LOCATION,
+        g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseKeyLoc", 
+                           0, G_MAXUINT, SKEY_LOC_INVALID, G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class, PROP_LOADED,
+        g_param_spec_uint ("loaded", "Loaded Information", "Which parts of the key are loaded. See SeahorseKeyLoaded", 
+                           0, G_MAXUINT, SKEY_INFO_NONE, G_PARAM_READABLE));
+
+    signals[CHANGED] = g_signal_new ("changed", G_OBJECT_CLASS_TYPE (gobject_class),
+        G_SIGNAL_RUN_LAST,  G_STRUCT_OFFSET (SeahorseKeyClass, changed),
+        NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+        
+    /* Some special fields */
+    special_properties[PROP_SPECIAL_DISPLAY_NAME] = g_quark_from_static_string ("display-name");
+    special_properties[PROP_SPECIAL_CN] = g_quark_from_static_string ("cn");
+}
+
+/* -----------------------------------------------------------------------------
+ * PUBLIC 
+ */
+
 /**
  * seahorse_key_destroy:
  * @skey: #SeahorseKey to destroy
@@ -219,28 +218,30 @@ seahorse_key_destroy (SeahorseKey *skey)
 void
 seahorse_key_changed (SeahorseKey *skey, SeahorseKeyChange change)
 {
-	g_return_if_fail (skey != NULL && SEAHORSE_IS_KEY (skey));
-	g_signal_emit (G_OBJECT (skey), signals[CHANGED], 0, change);
+    g_return_if_fail (skey != NULL && SEAHORSE_IS_KEY (skey));
+    g_signal_emit (G_OBJECT (skey), signals[CHANGED], 0, change);
 }
 
-const gchar*      
+GQuark
 seahorse_key_get_keyid (SeahorseKey *skey)
 {
-	g_return_val_if_fail (skey && SEAHORSE_IS_KEY (skey), NULL);
+    g_return_val_if_fail (skey && SEAHORSE_IS_KEY (skey), 0);
     return skey->keyid;
 }    
 
 const gchar*      
 seahorse_key_get_short_keyid (SeahorseKey *skey)
 {
+    const gchar *id;
     guint l;
     
-	g_return_val_if_fail (skey && SEAHORSE_IS_KEY (skey), NULL);
+    g_return_val_if_fail (skey && SEAHORSE_IS_KEY (skey), NULL);
+    id = g_quark_to_string (skey->keyid);
     
-    l = strlen (skey->keyid);
+    l = strlen (id);
     if (l > 8)
-        return skey->keyid + l - 8;
-    return skey->keyid;
+        return id + l - 8;
+    return id;
 }
 
 
@@ -270,7 +271,7 @@ seahorse_key_get_ktype (SeahorseKey *skey)
 SeahorseKeyEType
 seahorse_key_get_etype (SeahorseKey *skey)
 {
-    g_return_val_if_fail (SEAHORSE_IS_KEY (skey), SKEY_INVALID);
+    g_return_val_if_fail (SEAHORSE_IS_KEY (skey), SKEY_ETYPE_NONE);
     return skey->etype;
 }
 /**
@@ -291,7 +292,7 @@ seahorse_key_get_loaded (SeahorseKey *skey)
 SeahorseKeyLoc 
 seahorse_key_get_location (SeahorseKey *skey)
 {
-    g_return_val_if_fail (SEAHORSE_IS_KEY (skey), SKEY_LOC_UNKNOWN);
+    g_return_val_if_fail (SEAHORSE_IS_KEY (skey), SKEY_LOC_INVALID);
     return skey->location;
 }
 
@@ -385,3 +386,76 @@ seahorse_key_get_expires (SeahorseKey *skey)
     g_object_get (skey, "expires", &expires, NULL);
     return expires;
 }
+
+guint
+seahorse_key_get_length (SeahorseKey *skey)
+{
+    guint length;
+    g_object_get (skey, "length", &length, NULL);
+    return length;
+}
+
+const gchar*
+seahorse_key_get_stock_id (SeahorseKey *skey)
+{
+    const gchar* stock_id;
+    g_object_get (skey, "stock-id", &stock_id, NULL);
+    return stock_id;
+}
+
+void
+seahorse_key_set_preferred (SeahorseKey *skey, SeahorseKey *preferred)
+{
+    /* 
+     * We don't ref, because the SeahorseContext takes care of 
+     * keeping things sane. 
+     */
+    
+    if (preferred != skey->preferred) {
+        skey->preferred = preferred;
+        seahorse_key_changed (skey, SKEY_CHANGE_PREFERRED);
+    }
+}
+
+
+gboolean
+seahorse_key_lookup_property (SeahorseKey *skey, guint uid, const gchar *field, GValue *value)
+{
+    GParamSpec *spec;
+    GQuark qfield;
+    gchar *name;
+    
+    /* Special UID fields */
+    if (uid > 0) {
+
+        qfield = g_quark_from_string (field);
+        if (qfield == special_properties[PROP_SPECIAL_DISPLAY_NAME]) {
+            name = seahorse_key_get_name (skey, uid);
+            g_value_init (value, G_TYPE_STRING);
+            g_value_take_string (value, name ? name : g_strdup (""));
+            return TRUE;
+
+        } else if (qfield == special_properties[PROP_SPECIAL_CN]) {
+            name = seahorse_key_get_name_cn (skey, uid);
+            g_value_init (value, G_TYPE_STRING);
+            g_value_take_string (value, name ? name : g_strdup (""));
+            return TRUE;
+        }
+    }
+
+    spec = g_object_class_find_property (G_OBJECT_GET_CLASS (skey), field);
+    if (!spec) 
+        return FALSE;
+
+    g_value_init (value, spec->value_type);
+    g_object_get_property (G_OBJECT (skey), field, value);
+    return TRUE; 
+}
+
+const gchar* 
+seahorse_key_get_rawid (GQuark keyid)
+{
+    const gchar* id = g_quark_to_string (keyid);
+    const gchar* t = strchr (id, ':');
+    return t ? t + 1 : id;
+}
