@@ -53,6 +53,7 @@
  *   trust: (SeahorseValidity) Trust for the key.
  *   expires: (gulong) Date this key expires or 0.
  *   length: (guint) The length of the key in bits.
+ *   stock-id: (gpointer/string) The icon stock id
  */
  
 #ifndef __SEAHORSE_KEY_H__
@@ -213,6 +214,8 @@ gulong            seahorse_key_get_expires        (SeahorseKey        *skey);
 
 guint             seahorse_key_get_length         (SeahorseKey        *skey);
 
+const gchar*      seahorse_key_get_stock_id       (SeahorseKey        *skey);
+
 void              seahorse_key_set_preferred      (SeahorseKey        *skey,
                                                    SeahorseKey        *preferred);
 
