@@ -31,6 +31,8 @@
 #include "seahorse-vfs-data.h"
 #include "seahorse-util.h"
 
+#define PROGRESS_BLOCK  16 * 1024
+
 /* The current/last VFS operation */
 typedef enum _VfsAsyncOp
 {
@@ -53,6 +55,9 @@ VfsAsyncState;
 
 /* A handle for VFS work */
 typedef struct _VfsAsyncHandle {
+    
+    gpgme_data_t gdata;             /* A pointer to the outside gpgme_data_t handle */
+    
     gchar *uri;                     /* The URI we're operating on */
     GnomeVFSAsyncHandle* handle;    /* Handle for operations */
     GtkWidget* widget;              /* Widget for progress and cancel */
@@ -63,6 +68,12 @@ typedef struct _VfsAsyncHandle {
     GnomeVFSResult result;          /* Result of the current operation */
     gpointer buffer;                /* Current operation's buffer */
     GnomeVFSFileSize processed;     /* Number of bytes processed in current op */
+    
+    GnomeVFSFileSize last;          /* Last update sent about number of bytes */
+    GnomeVFSFileSize total;         /* Total number of bytes read or written */
+    
+    SeahorseVfsProgressCb progcb;   /* Progress callback */
+    gpointer userdata;              /* User data for progress callback */
 } VfsAsyncHandle;
 
 static void vfs_data_cancel(VfsAsyncHandle *ah);
@@ -187,6 +198,8 @@ vfs_data_open_done(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     
         ah->result = result;
         ah->state = VFS_ASYNC_READY;
+        ah->total = 0;
+        ah->last = 0;
     }
 }
 
@@ -194,9 +207,9 @@ vfs_data_open_done(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 static void
 vfs_data_open_helper (VfsAsyncHandle *ah, gboolean write)
 {
-    g_return_if_fail (ah->handle == NULL);
-    g_return_if_fail (ah->uri != NULL);
-    g_return_if_fail (ah->state == VFS_ASYNC_READY);
+    g_assert (ah->handle == NULL);
+    g_assert (ah->uri != NULL);
+    g_assert (ah->state == VFS_ASYNC_READY);
     
     if (write) {
         /* Note that we always overwrite the file */
@@ -208,7 +221,7 @@ vfs_data_open_helper (VfsAsyncHandle *ah, gboolean write)
     }
 
     ah->state = VFS_ASYNC_PROCESSING;
-    ah->operation = VFS_OP_OPENING;        
+    ah->operation = VFS_OP_OPENING;
 }
 
 /* Open the given URI */
@@ -218,7 +231,7 @@ vfs_data_open(const gchar* uri, gboolean write, gboolean delayed)
     VfsAsyncHandle* ah;
 
     /* We only support delayed opening of write files */
-    g_return_val_if_fail (write || !delayed, NULL);
+    g_assert (write || !delayed);
     
     ah = g_new0(VfsAsyncHandle, 1);
     ah->state = VFS_ASYNC_READY;
@@ -248,6 +261,11 @@ vfs_data_read_done(GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer
         ah->result = result;
         ah->processed = bytes_read;
         ah->state = VFS_ASYNC_READY;
+        ah->total += bytes_read;
+        
+        /* Call progress callback if setup */
+        if (ah->progcb && ah->total >= ah->last + PROGRESS_BLOCK)
+            (ah->progcb) (ah->gdata, ah->total, ah->userdata);
     }
 }    
     
@@ -258,7 +276,7 @@ vfs_data_read(void *handle, void *buffer, size_t size)
     VfsAsyncHandle* ah = (VfsAsyncHandle*)handle;
     ssize_t sz = 0;
     
-    g_return_val_if_fail (ah->handle != NULL, -1);
+    g_assert (ah->handle != NULL);
 
     /* Just in case we have an operation, like open */
     if(!vfs_data_wait_results(ah, TRUE))
@@ -302,6 +320,11 @@ vfs_data_write_done(GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gconstpo
         ah->result = result;
         ah->processed = bytes_written;
         ah->state = VFS_ASYNC_READY;
+        ah->total += bytes_written;
+        
+        /* Call progress callback if setup */
+        if (ah->progcb && ah->total >= ah->last + PROGRESS_BLOCK)
+            (ah->progcb) (ah->gdata, ah->total, ah->userdata);
     }
 }    
 
@@ -468,15 +491,16 @@ static struct gpgme_data_cbs vfs_data_cbs =
 
 /* -----------------------------------------------------------------------------
  */
- 
+
 /* Create a data on the given uri, remote uris get gnome-vfs backends,
  * local uris get normal file access. */
-gpgme_data_t
-seahorse_vfs_data_create(const gchar *uri, guint mode, gpg_error_t* err) 
+static gpgme_data_t
+create_vfs_data (const gchar *uri, guint mode, SeahorseVfsProgressCb progcb,
+                 gpointer userdata, gpg_error_t* err) 
 {
     gpgme_error_t gerr;
     gpgme_data_t ret = NULL;
-    void* handle = NULL;
+    VfsAsyncHandle* handle = NULL;
     char* ruri;
     
     if (!err)
@@ -487,11 +511,16 @@ seahorse_vfs_data_create(const gchar *uri, guint mode, gpg_error_t* err)
     handle = vfs_data_open (ruri, mode & SEAHORSE_VFS_WRITE, 
                                   mode & SEAHORSE_VFS_DELAY);
     if (handle) {
+        
         *err = gpgme_data_new_from_cbs (&ret, &vfs_data_cbs, handle);
         if (!GPG_IS_OK (*err)) {
             vfs_data_cbs.release (handle);
             ret = NULL;
         }
+        
+        handle->progcb = progcb;
+        handle->userdata = userdata;
+        handle->gdata = ret;
     }
     
     g_free (ruri);
@@ -499,3 +528,192 @@ seahorse_vfs_data_create(const gchar *uri, guint mode, gpg_error_t* err)
     return ret;
 }
 
+
+gpgme_data_t 
+seahorse_vfs_data_create (const gchar *uri, guint mode, GError **err)
+{
+    return seahorse_vfs_data_create_full (uri, mode, NULL, NULL, err);
+}
+
+gpgme_data_t 
+seahorse_vfs_data_create_full (const gchar *uri, guint mode, SeahorseVfsProgressCb progcb,
+                               gpointer userdata, GError **err)
+{
+    gpgme_data_t data;
+    gpgme_error_t gerr;
+    
+    g_return_val_if_fail (!err || !*err, NULL);
+    data = create_vfs_data (uri, mode, progcb, userdata, &gerr);
+    if (!data)
+        seahorse_util_gpgme_to_error (gerr, err);
+    return data;
+}
+
+gpgme_data_t
+seahorse_vfs_data_create_gerr (const gchar *uri, guint mode, gpg_error_t* err) 
+{
+    return create_vfs_data (uri, mode, NULL, NULL, err);
+}
+
+gboolean
+seahorse_vfs_set_file_contents (const gchar *uri, const gchar *text, gint len, 
+                                GError **err)
+{
+    gpgme_data_t data;
+    gboolean ret;
+    
+    data = seahorse_vfs_data_create (uri, SEAHORSE_VFS_WRITE, err);
+    if (!data)
+        return FALSE;
+    
+    ret = seahorse_vfs_data_write_all (data, (void*)text, len, err);
+    
+    gpgme_data_release (data);
+    return ret;
+}
+
+gboolean 
+seahorse_vfs_data_write_all (gpgme_data_t data, const void* buffer, gint len, GError **err)
+{
+    guchar *text = (guchar*)buffer;
+    gint written;
+    
+    if (len < 0)
+        len = strlen ((gchar*)text);
+    
+    while (len > 0) {
+        written = gpgme_data_write (data, (void*)text, len);
+        if (written < 0) {
+            if (errno == EAGAIN || errno == EINTR)
+                continue;
+            g_set_error (err, G_FILE_ERROR, g_file_error_from_errno (errno), 
+                         "%s", strerror (errno));
+            return FALSE;
+        }
+        
+        len -= written;
+        text += written;
+    }
+    
+    return TRUE;
+}
+
+/* -----------------------------------------------------------------------------
+ * MULTIPLE CONCATED FILES 
+ */
+
+/* Called by gpgme to read data */
+static ssize_t
+multi_data_read (void *handle, void *buffer, size_t size)
+{
+    gpgme_data_t data = NULL;
+    ssize_t sz = 0;
+    GList *datas, *l;
+    guchar *buf;
+
+    datas = (GList*)handle;
+    buf = (guchar*)buffer;
+    
+    /* Find first non-null data thingy */
+    for (l = datas; l; l = g_list_next (l)) {
+        data = (gpgme_data_t)l->data;
+        if (data)
+            break;
+    }
+    
+    while (size > 0 && data) {
+        sz = gpgme_data_read (data, buf, size);
+        
+        /* An error */
+        if (sz < 0)
+            return sz;
+        
+        if (sz > size)
+            sz = size;
+        
+        /* Free current data, get next one */
+        if (sz < size) {
+            l->data = NULL;
+            gpgmex_data_release (data);
+            l = g_list_next (l);
+            data = (gpgme_data_t)(l ? l->data : NULL);
+        }
+        
+        /* Read later in buffer */
+        size -= sz;
+        buf += sz;
+    }
+    
+    return sz;
+}
+
+/* Called by gpgme to write data */
+static ssize_t
+multi_data_write (void *handle, const void *buffer, size_t size)
+{
+    /* No writing */
+    errno = EBADF;
+    return -1;
+}
+
+/* Called from gpgme to seek a file */
+static off_t
+multi_data_seek(void *handle, off_t offset, int whence)
+{
+    /* No real seeking */
+    return offset;
+}
+
+/* Called by gpgme to close a file */
+static void
+multi_data_release(void *handle)
+{
+    GList *datas = (GList*)handle;
+    GList *l;
+
+    for (l = datas; l; l = g_list_next (l))
+        gpgmex_data_release ((gpgme_data_t)l->data);
+    g_list_free (datas);
+}
+
+static struct gpgme_data_cbs multi_data_cbs = 
+{
+    multi_data_read,
+    multi_data_write,
+    multi_data_seek,
+    multi_data_release
+};
+
+gpgme_data_t
+seahorse_vfs_data_read_multi (const gchar **uris, GError **err)
+{
+    gpgme_error_t gerr;
+    gpgme_data_t data;
+    GList *datas = NULL;
+    const gchar **u;
+    
+    for (u = uris; *u; u++) {
+        if(!(*u)[0])
+            continue;
+        data = seahorse_vfs_data_create (*u, SEAHORSE_VFS_READ, err);
+        if (!data)
+            break;
+        datas = g_list_prepend (datas, data);
+    }
+    
+    /* Failed somewhere */
+    if (!data) {
+        multi_data_release (datas);
+        return NULL;
+    }
+    
+    datas = g_list_reverse (datas);
+    gerr = gpgme_data_new_from_cbs (&data, &multi_data_cbs, datas);
+    if (!GPG_IS_OK (gerr)) {
+        seahorse_util_gpgme_to_error (gerr, err);
+        multi_data_release (datas);
+        return NULL;
+    }
+
+    return data;
+}
