@@ -34,17 +34,15 @@
 typedef struct _SeahorseKeyManagerStore SeahorseKeyManagerStore;
 typedef struct _SeahorseKeyManagerStoreClass SeahorseKeyManagerStoreClass;
 	
-struct _SeahorseKeyManagerStore
-{
-	SeahorseKeyStore	parent;
+struct _SeahorseKeyManagerStore {
+    SeahorseKeyStore    parent;
 };
 
-struct _SeahorseKeyManagerStoreClass
-{
-	SeahorseKeyStoreClass	parent_class;
+struct _SeahorseKeyManagerStoreClass {
+    SeahorseKeyStoreClass	parent_class;
 };
 
-SeahorseKeyStore*	seahorse_key_manager_store_new	(SeahorseKeySource	*sksrc,
+SeahorseKeyStore*	seahorse_key_manager_store_new	(SeahorseKeyset	    *skset,
 							                         GtkTreeView        *view);
 
 /* Drag and trop target types */
